#!/usr/bin/perl -w

# Application Checker
# Configuration Page Module (tests_conf.pl)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my @spec_versions_list = ();
my %spec_profiles_list = ();
my $default_spec_version;
if ($appchecker_branch{'STD'} eq 'LSB') {
	@spec_versions_list = qw/4.0 3.2 3.1 3.0/;
	$default_spec_version = '4.0';
	%spec_profiles_list = ('core,c++' => 'Core &amp; C++', 'core,c++,desktop' => 'Core &amp; C++ &amp; Desktop');
}
else {
	@spec_versions_list = qw/2.0/;
	$default_spec_version = '2.0';
}

my %archs_list = ();

my @error_msg = ();
my @warn_msg = ();

my %default_profile_data = (
	'application' => '',
	'components' => '',
	'architecture' => '',
	'spec_version' => $default_spec_version,
	'spec_profile' => 'core,c++,desktop',
	'show_hidden_files' => '0',
	'last_opened_dir' => '/',
	'comments' => ''
);

my %profile_data = %default_profile_data;

# Determine architecture of the appchk binary to use it as the only possible value (yet).
$default_profile_data{'architecture'} = guess_test_arch($CONFIG{'TESTS_DIR'}.'/'.lc($appchecker_branch{'STD'}).'appchk');
if ($default_profile_data{'architecture'} eq 'unknown') {
	if ($appchecker_branch{'STD'} eq 'LSB') {
		push @warn_msg, 'Could not detect the lsbappchk architecture:<br />the package <b>lsb-appchk</b> is corrupted or not installed.<br />Try reinstalling Linux AppChecker.';
	}
	else {
		push @warn_msg, 'Could not detect the moblinappchk architecture:<br />the package <b>moblin-app-checker</b> is probably corrupted.<br />Try reinstalling Moblin AppChecker.';
	}
	$default_profile_data{'architecture'} = 'x86';
}
$profile_data{'architecture'} = $default_profile_data{'architecture'};

my $error_text = '';
my $warn_text = '';
my $empty_output = 1;

if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
	system(esc_cmdline("mkdir -p «$SERVER_PARAM{'APP_DATA'}/profiles»"));
	if (!-d $SERVER_PARAM{'APP_DATA'}.'/profiles') {
		push @error_msg, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = '';
my $create_session_error = 0;

if (!$_COOKIE{'session_id'} or !-f $SERVER_PARAM{'APP_DATA'}.'/profiles/~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$_COOKIE{'session_id'}) {
	my $i = time();
	while (-f $SERVER_PARAM{'APP_DATA'}.'/profiles/~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$i) {
		++$i;
	}
	$_COOKIE{'session_id'} = $i;
	$profile_name = '~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$i;
	if (open(FILE, '>'.$SERVER_PARAM{'APP_DATA'}."/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		push @error_msg, "Cannot create the session profile '$profile_name'<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = '~session.'.$SERVER_PARAM{'PEER_IP'}.'.'.$_COOKIE{'session_id'};
}

if (open(FILE, $SERVER_PARAM{'APP_DATA'}."/profiles/$profile_name")) {
	my @comments = ();
	my @components = ();
	while (<FILE>) {
		s/[\r\n]//g;
		if (/^APPLICATION:\s+(.*)$/) {
			$profile_data{'application'} = $1;
		}
		elsif (/^COMPONENT:\s+(.*)$/) {
			push @components, $1;
		}
		elsif (/^SPEC_VERSION:\s+(.*)$/) {
			$profile_data{'spec_version'} = $1;
		}
		elsif (/^SPEC_PROFILE:\s+(.*)$/ and ($appchecker_branch{'STD'} eq 'LSB')) {
			$profile_data{'spec_profile'} = $1;
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{'architecture'} = guess_platform($1);
		}
		elsif (/^SHOW_HIDDEN:\s+(.*)$/) {
			$profile_data{'show_hidden_files'} = $1;
		}
		elsif (/^LAST_DIR:\s+(.*)$/) {
			$profile_data{'last_opened_dir'} = $1;
		}
		elsif (/^COMMENTS: (.*)$/) {
			push @comments, $1;
		}
	}
	close(FILE);
	$profile_data{'components'} = join("\n", @components);
	$profile_data{'comments'}   = join("\n", @comments);
}
elsif (!$create_session_error) {
	push @error_msg, "Cannot read the session profile data: $profile_name!<br />$!";
}

if (system('readelf --help >/dev/null 2>&1') != 0) {
	push @error_msg, "Can't find <b>readelf</b>. This utility is required for application analysis.<br />It is usually a part of the <b>binutils</b> package. Please install it first.";
}

if ($_COOKIE{'last_error_text'}) {
	push @error_msg, $_COOKIE{'last_error_text'};
}

# Set cookie expiration to 7 days since now
my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time() + 7*86400);
my @wday_names = qw/Sunday Monday Tuesday Wednesday Thursday Friday Saturday/;
my @mon_names = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
# Format date like follows: Tuesday, 31-Dec-19 21:00:00 GMT
my $fmt_date = sprintf('%s, %d-%s-%d %02d:%02d:%02d GMT', $wday_names[$wday], $mday, $mon_names[$mon], 1900 + $year, $hour, $min, $sec);
print 'HTTP/1.0 200 OK' . CRLF;
print 'Content-type: text/html' . CRLF;
print 'Set-Cookie: session_id=' . $_COOKIE{'session_id'} . '; expires=' . $fmt_date . '; path=/;' . CRLF;
print 'Set-Cookie: last_error_text=; path=/;' . CRLF . CRLF;

print_header('Application Check', 'conf');

print <<DATA;

<script language="javascript" type="text/javascript">
// <![CDATA[
var try_num = 0;
function addCookieError() {
	error_text = 'It seems that you have cookies turned off in your browser.<br />Please turn them on or some important functions will fail.';
	++try_num;
	if (err_text && err_area) {
		error_text = '<font color="darkblue">' + error_text + '</font>';
		if (err_text.innerHTML != '')
			error_text = '<br />' + error_text;
		err_text.innerHTML += error_text;
		err_area.style.display = '';
	}
	else {
		// Wait for 1 second till the page is loaded
		if (try_num < 3)
			setTimeout('addCookieError()', 300);
		else
			alert(error_text.replace(/<br\\s*\\/>/g, '\\n'));  // Something wrong, giving up.
	}
}

if (document.cookie == '') {
	setTimeout('addCookieError()', 300);
}
// ]]>
</script>
DATA

if ((scalar(@error_msg) != 0) or (scalar(@warn_msg) != 0)) {
	$error_text = '<ul>';
	foreach (@error_msg) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msg) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= '</ul>';
}

my $table_spacer = '<td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>';

print show_error_dlg($error_text);

print <<DATA;

<div id="tooltip" style="display: none; background-color: #ffffd0; border: solid 1px black; position: absolute; z-index: 1000;">
<table cellpadding="0" cellspacing="0" border="0" style="width: 17em;">
  <tr style="background-color: #d0d0d0; color: black;">
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/environment-spacer.gif" width="7" height="1" alt="" /></td>
    <td align="center" width="100%" style="border-bottom: solid 1px black;"><b>Information</b></td>
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/close.png" onclick="javascript:initDialog(null);" width="20" height="20" title="Close" alt="X" style="cursor: pointer;" /></td>
  </tr>
  <tr>
    <td colspan="3" style="padding: 5px; white-space: nowrap;" id="tooltip_text"></td>
  </tr>
</table>
</div>

<table cellpadding="0" cellspacing="0" border="0" style="display: none; position: absolute; z-index: 999;" id="shadow">
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_tl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_tc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_tr.png') no-repeat;"></td>
  </tr>
  <tr>
    <td width="14" style="background: url('images/shadow_ml.png') repeat-y;"></td>
    <td style="background: url('images/shadow_mc.png') repeat;" id="shadow_center"></td>
    <td width="14" style="background: url('images/shadow_mr.png') repeat-y;"></td>
  </tr>
  <tr>
    <td width="14" height="14" style="background: url('images/shadow_bl.png') no-repeat;"></td>
    <td height="14" style="background: url('images/shadow_bc.png') repeat-x;"></td>
    <td width="14" height="14" style="background: url('images/shadow_br.png') no-repeat;"></td>
  </tr>
</table>

<div id="ajax_loading" style="display: none;"><div align="center">
Please wait&hellip;<br />
<img src="images/environment-spacer.gif" width="1" height="5" alt="" /><br />
<img src="themes/$appchecker_branch{'DIR'}/images/ajax_progress.gif" width="16" height="16" alt="" /><br />
<img src="images/environment-spacer.gif" width="1" height="7" alt="" /><br />
<input type="button" name="ajax_abort" id="ajax_abort" value="Abort" onclick="javascript:onAbort();" />
</div></div>

<h1>Specify Your Application</h1>
<form method="post" action="tests_conf_save.pl" name="configure_test" id="configure_test">
<input type="hidden" name="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="form_action" id="form_action" value="" />
<input type="hidden" name="show_hidden_files" id="show_hidden_files" value="$default_profile_data{show_hidden_files}" />
<input type="hidden" name="last_opened_dir" id="last_opened_dir" value="$default_profile_data{last_opened_dir}" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td valign="top" style="white-space: nowrap;">Name:&nbsp;<font color="red">*</font>&nbsp;<a href="tests_help.pl#app_name" onclick="javascript:show_hide_tooltip(this, 'app_name', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></td>
          $table_spacer
          <td width="100%" colspan="2"><input type="text" name="application" id="application" value="$default_profile_data{application}" style="width: 40em;" onfocus="javascript:style.borderColor=input_border_color;" /></td>
        </tr>
        <tr>
          <td valign="top" style="white-space: nowrap;">Components:&nbsp;<font color="red">*</font>&nbsp;<a href="tests_help.pl#profile_management" onclick="javascript:show_hide_tooltip(this, 'file_formats', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></td>
          $table_spacer
          <td width="100%" colspan="2"><textarea name="components" id="components" rows="7" cols="58" style="width: 40em;" onfocus="javascript:style.borderColor=input_border_color;">$default_profile_data{components}</textarea></td>
        </tr>
        <tr>
          <td width="100%" colspan="4" align="right"><input type="button" name="browse_for_file_but" value="Select Application Components&hellip;" style="width: 18em;" onclick="javascript:select_components();" /></td>
        </tr>
        <tr>
          <td colspan="4" valign="bottom" height="15"><hr /></td>
        </tr>
        <tr>
          <td colspan="4" style="border: none; background: #ffffff; cursor: pointer;" onclick="javascript:initDialog(null);showHideDetails();">&nbsp;<img src="themes/$appchecker_branch{'DIR'}/images/arr_r.png" alt="" id="block_arrow_img" width="12" height="12" />&nbsp;&nbsp;Additional options</td>
        </tr>
        <tr>
          <td colspan="4" id="additional_data" style="border: dashed 1px gray; display: none;">
            <table border="0" cellpadding="2" cellspacing="2" width="100%">
              <tr>
                <td valign="middle" colspan="4">
                  <table border="0" cellpadding="0" cellspacing="0" width="100%">
                    <tr>
                      <td>Architecture:</td>
                      <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                      <td>
                        <select name="architecture" id="architecture" style="width: 6em;" disabled="disabled">
DATA

my @archs;

if ($default_profile_data{'architecture'} =~ m/unknown/i) {
	@archs = qw/x86 x86-64 IA64 PPC32 PPC64 S390 S390X/;
}
elsif ($default_profile_data{'architecture'} =~ m/x86-64/i) {
	@archs = qw/x86 x86-64/;
}
elsif ($default_profile_data{'architecture'} =~ m/IA64/i) {
	@archs = qw/x86 IA64/;
}
elsif ($default_profile_data{'architecture'} =~ m/PPC64/i) {
	@archs = qw/PPC32 PPC64/;
}
elsif ($default_profile_data{'architecture'} =~ m/S390X/i) {
	@archs = qw/S390 S390X/;
}
else {
	@archs = ($default_profile_data{'architecture'});
}

foreach (sort @archs) {
	if ($_ eq $default_profile_data{'architecture'}) {
		print "                          <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "                          <option value=\"$_\">$_</option>\n";
	}
}

my $subtable_right_fill = (($appchecker_branch{'STD'} eq 'LSB') ? '' : "                            <td width=\"100%\"></td>\n");
my $profiles_block;
if ($appchecker_branch{'STD'} eq 'LSB') {
	$profiles_block = <<DATA;
                      <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                      <td>$appchecker_branch{'STD'}&nbsp;Profile:</td>
                      <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                      <td width="100%">
                        <select name="spec_profile" id="spec_profile" style="width: 15em;">
DATA

	foreach (sort keys %spec_profiles_list) {
		if ($_ eq $default_profile_data{'spec_profile'}) {
			$profiles_block .= "                          <option value=\"$_\" selected=\"selected\">$spec_profiles_list{$_}</option>\n";
		}
		else {
			$profiles_block .= "                          <option value=\"$_\">$spec_profiles_list{$_}</option>\n";
		}
	}

	$profiles_block .= <<DATA;
                        </select>
                      </td>
DATA
}
else {
	$profiles_block = '';
}

print <<DATA;
                        </select>
                      </td>
                      <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                      <td width="100%" align="center">
                        <table cellpadding="0" cellspacing="0" border="0">
                          <tr>
                            <td>$appchecker_branch{'STD'}&nbsp;Spec:</td>
                            <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                            <td>
                              <select name="spec_version" id="spec_version" style="width: 6em;" onchange="javascript:control_profile_list();">
DATA

foreach (@spec_versions_list) {
	if ($_ eq $default_profile_data{'spec_version'}) {
		print "                                <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "                                <option value=\"$_\">$_</option>\n";
	}
}

print <<DATA;
                              </select>
                            </td>
$subtable_right_fill                          </tr>
                        </table>
                      </td>
$profiles_block                    </tr>
                  </table>
                </td>
              </tr>
              <tr>
                <td colspan="4"><img src="images/environment-spacer.gif" width="1" height="5" border="0" alt="" /></td>
              </tr>
              <tr>
                <td valign="top">Comments:&nbsp;<a href="tests_help.pl#comments" onclick="javascript:show_hide_tooltip(this, 'comments', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></td>
                $table_spacer
                <td colspan="2" align="right"><textarea name="comments" id="comments" rows="5" cols="58" style="width: 40em;">$default_profile_data{'comments'}</textarea></td>
              </tr>
              <tr>
                <td colspan="4"><img src="images/environment-spacer.gif" width="1" height="3" border="0" alt="" /></td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td colspan="4" align="right"><input type="reset" name="reset_button" id="reset_button" value="Clear Form" style="width: 11em;" onclick="javascript:reset_form(); return false;" />&nbsp;&nbsp;&nbsp;<input type="submit" name="run_tests" id="run_tests_button" value="Run the Test" style="width: 11em; font-weight: bold;" onclick="javascript:return do_action('run_tests');" /></td>
        </tr>
      </table>
    </td>
    <td style="border-right: dashed 1px gray;"><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
DATA

my $profiles_options = '';
my @profiles_list;
if (opendir(DIR, $SERVER_PARAM{'APP_DATA'}.'/profiles')) {
	@profiles_list = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@profiles_list) {
		$profiles_options .= "                <option value=\"$_\">$_</option>\n";
	}
	closedir(DIR);
}

my $disabled_list = '';
if ($profiles_options eq '') {
	$profiles_options = "                <option value=\"\">&lt;No profiles present&gt;</option>\n";
	$disabled_list = ' disabled="disabled"';
}

my $profiles_list_num = scalar(@profiles_list);
if ($profiles_list_num < 2) {
	$profiles_list_num = 2;
}
elsif ($profiles_list_num > 6) {
	$profiles_list_num = 6;
}

my $array_profiles = join(', ', map("'".esc_js($_)."'", @profiles_list));
my $packager_name = ($is_packager_rpm ? 'RPM' : 'DEB');

print <<DATA;
    <td valign="top" width="50%">
      <h2>User Profile Management&nbsp;&nbsp;&nbsp;<a href="tests_help.pl#profile_management" onclick="javascript:show_hide_tooltip(this, 'profile_management', event); return false;"><img src="themes/$appchecker_branch{'DIR'}/images/question.png" width="16" height="16" alt="?" /></a></h2>
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="3">Select the profile or type its name</td>
        </tr>
        <tr>
          <td colspan="3">
            <table border="0" cellpadding="0" cellspacing="0"><tr>
              <td style="vertical-align: middle;">
                <input type="text" name="edit_profile_name" id="edit_profile_name" value="" style="width: 18em;" onkeypress="javascript:return onKeyPressEdit(event);" onfocus="javascript:style.borderColor=input_border_color;" />
              </td>
              <td style="vertical-align: middle;">
                <img src="images/drop_down.png" alt="v" width="16" height="20" id="drop_down_button" style="cursor: pointer;" onclick="javascript:onClickArrow();" />
              </td>
            </tr></table>
            <div id="hidden_div" style="position: absolute; z-index: +1; display: none;">
              <select name="list_profile_name" id="list_profile_name" size="$profiles_list_num" style="font-family: monospace; font-size: 100%; width: 19.7em;"$disabled_list onclick="javascript:onClickList();" onkeypress="javascript:return onKeyPressList(event);">
$profiles_options              </select>
            </div>
          </td>
        </tr>
        <tr>
          <td><input type="button" name="save_profile_button" id="save_profile_button" value="Save" style="width: 5.8em;" onclick="javascript:onSave();" /></td>
          <td><input type="button" name="load_profile_button" id="load_profile_button" value="Load" style="width: 5.8em;" onclick="javascript:onLoad();" /></td>
          <td width="100%"><input type="button" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 5.8em;" onclick="javascript:onDelete();" /></td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</form>
<script language="javascript" type="text/javascript">
// <![CDATA[
// Preload some images that are not initially visible
preload_images(new Array('expand.png', 'collapse.png', 'themes/$appchecker_branch{'DIR'}/images/arr_d.png', 'drop_down_pressed.png', 'close.png', 'themes/$appchecker_branch{'DIR'}/images/ajax_progress.gif', 'shadow_bc.png', 'shadow_bl.png', 'shadow_br.png', 'shadow_mc.png', 'shadow_ml.png', 'shadow_mr.png', 'shadow_tc.png', 'shadow_tl.png', 'shadow_tr.png'));

// Cache the necessary elements of the page
var edit_profile_name = document.getElementById('edit_profile_name');
var list_profile_name = document.getElementById('list_profile_name');
var hidden_div = document.getElementById('hidden_div');
var drop_down_button = document.getElementById('drop_down_button');
var app_name = document.getElementById('application');
var dialog_shadow = document.getElementById('shadow');
var dialog_shadow_center = document.getElementById('shadow_center');
if (!list_profile_name || !edit_profile_name || !hidden_div || !drop_down_button || !app_name || !dialog_shadow || !dialog_shadow_center)
	alert('Internal error: Cannot find essential form fields! JavaScript will not work correctly.\\nPlease contact the authors.');

var input_border_color = ((navigator.appName == 'Konqueror') ? '#cccccc' : '');
var profiles_list = new Array($array_profiles);

var just_shown_dialog = false;          // Indicates that the dialog was just shown (to prevent its hiding from onBodyClick)
var dialog = null;                      // The dialog itself
var dialog_caller = null;               // Element which caused the dialog to appear
var dialog_caller_class = null;         // Normal class name of the dialog caller (to be restored when the dialog is closed)

// Hides the previous dialog (if it was shown) and prepare data for
// displaying the new one.
// CALLER - the HTML element which caused the dialog to appear and (possibly)
// will change its appearance because of that.
function initDialog(caller) {
	if (dialog_caller && dialog_caller_class)
		dialog_caller.className = dialog_caller_class;
	dialog_shadow.style.display = 'none';
	if (dialog)
		dialog.style.display = 'none';
	if (!caller || (caller == dialog_caller)) {
		dialog_caller = null;
		dialog_caller_class = null;
		return false;
	}
	else {
		dialog_caller = caller;
		dialog_caller_class = caller.className;
		return true;
	}
}

// Shows the dialog at the specified position and adjusts the shadow below it.
function showDialog(show_dlg, pos) {
	dialog = show_dlg;
	dialog.style.left = pos.x + 'px';
	dialog.style.top = pos.y + 'px';
	dialog.style.display = '';
	just_shown_dialog = true;
	dialog_shadow_center.width = dialog.offsetWidth - 15;
	dialog_shadow_center.height = dialog.offsetHeight - 15;
	dialog_shadow.style.left = pos.x + 'px';
	dialog_shadow.style.top = pos.y + 'px';
	dialog_shadow.style.display = '';
}

// Writes down all the profile data into the form
function apply_profile_data() {
	var profile_data = new Array();
DATA

foreach my $field (keys %profile_data) {
	my $data = esc_js($profile_data{$field});
	$data =~ s/\r?\n/\\n/g;
	print "\tprofile_data['$field'] = '$data';\n";
}

print <<DATA;
	for (var field in profile_data) {
		var elem = document.getElementById(field);
		if (elem)
			elem.value = profile_data[field];
	}
}

// Processing any significant action
function do_action(name) {
	var elem = document.getElementById('form_action');
	var pass = true;
	if (!elem) {
		alert("Internal error: Cannot find 'form_action' field!\\nPlease contact the authors.");
		return false;
	}
	if (name == 'run_tests') {          // Maybe, remove parameter 'name' at all...
		var msg = 'There are some problems detected:';
		if (!app_name || (app_name.value == '')) {
			app_name.style.borderColor = 'red';
			msg += '\\nThe application name is not specified.';
			pass = false;
		}
		else if (app_name.value.search(/[/\\\\<>|\\?\\*]/) != -1) {
			app_name.style.borderColor = 'red';
			msg += '\\nThe application name should contain only symbols that are allowed in file names.';
			pass = false;
		}
		var components_elem = document.getElementById('components');
		if (components_elem.value == '') {
			components_elem.style.borderColor = 'red';
			msg += '\\nPlease specify something to test.';
			pass = false;
		}
		else {
			var comps = components_elem.value.split('\\n');
			var bad_elems = new Array();
			for (var i = 0; i < comps.length; ++i) {
				comps[i] = comps[i].replace(/[\\n\\r]/g, '');
				comps[i] = comps[i].replace(/^\\s*(.*?)\\s*\$/, '\$1');
				if ((comps[i] != '') && (comps[i].indexOf('/') != 0) && (comps[i].indexOf('pkg:') != 0))
					bad_elems.push(comps[i]);
			}
			if (bad_elems.length != 0) {
				components_elem.style.borderColor = 'red';
				msg += '\\nAll items should be either absolute paths or installed packages.\\nThe following items are incorrect:\\n' + bad_elems.join('\\n');
				pass = false;
			}
		}
		if (!pass)
			alert(msg);
	}
	elem.value = name;
	return pass;
}

function reset_form() {
	if (confirm('Are you sure you want to clear all the fields?')) {
		document.configure_test.reset();
		control_profile_list();
	}
}

// Tooltip processing functions
var tooltip_texts = new Array();
tooltip_texts['app_name'] = 'Enter any name for your application<br />that will appear in the reports.';
tooltip_texts['comments'] = 'You can write any additional information<br />here for display on the <b>Results History</b> page<br />to help distinguish the results between different tests.';
tooltip_texts['file_formats'] = 'This field contains all components of the application:<ul style="line-height: 1.2em; margin-top: 0;"><li>Individual files</li><li>Whole directories</li><li>Installed $packager_name packages (prepended with <b>pkg:</b>)</li><li>RPM and DEB package files</li><li>TAR.GZ and TAR.BZ2 archives</li></ul><br />The archives will be automatically unpacked.<br />Only the files of the following types will be tested:<ul style="line-height: 1.2em; margin-top: 0;"><li>ELF (executables and shared objects)</li><li>Perl scripts</li><li>Python scripts</li><li>Shell scripts</li></ul>';
tooltip_texts['profile_management'] = 'This section allows you to save all the settings from<br />the left pane into user profiles and load them afterwards.';

function show_hide_tooltip(elem_parent, id, event) {
	if (!initDialog(elem_parent))
		return;
	var tt_window = document.getElementById('tooltip');
	if (tt_window) {
		var tt_text = document.getElementById('tooltip_text');
		if (tt_text) {
			if (tooltip_texts[id])
				tt_text.innerHTML = tooltip_texts[id] + '<br /><div align="right"><a href="tests_help.pl#' + id + '" target="_blank" onclick="javascript:initDialog(null);"><i>Details...</i></a></div>';
			else
				tt_text.innerHTML = '<font color="red"><b>Sorry, no information for this item was found!</b></font><br />Please write to <a href="mailto:lsb-appcheck-support\@linuxfoundation.org">lsb-appcheck-support\@linuxfoundation.org</a>.';
			var scrollTop = document.body.scrollTop;
			if (scrollTop == 0)
			{
				if (window.pageYOffset)
					scrollTop = window.pageYOffset;
				else
					scrollTop = (document.body.parentElement) ? document.body.parentElement.scrollTop : 0;
			}
			var scrollLeft = document.body.scrollLeft;
			if (scrollLeft == 0)
			{
				if (window.pageXOffset)
					scrollLeft = window.pageXOffset;
				else
					scrollLeft = (document.body.parentElement) ? document.body.parentElement.scrollLeft : 0;
			}
			var div_top = event.clientY + scrollTop + 10;
			var div_left = event.clientX + scrollLeft + 10;
			tt_window.style.display = '';
			var div_width = (tt_window.style.pixelWidth || tt_window.offsetWidth);
			var wnd_width = (document.body.clientWidth || window.innerWidth);
			if (div_left + div_width > wnd_width + scrollLeft)
				div_left = wnd_width + scrollLeft - div_width;
			if (div_left < scrollLeft)
				div_left = scrollLeft;
			showDialog(tt_window, {x: div_left, y: div_top});
		}
	}
}

// Editable drop-down list functions
var VK_ESC  = 27;
var VK_CR   = 13;
var VK_LF   = 10;
var VK_DOWN = 40;
var VK_TAB  = 9;

var just_shown_list = false;

function show_list() {
	hidden_div.style.display = '';
	drop_down_button.setAttribute('src', 'images/drop_down_pressed.png');
	just_shown_list = true;
}

function hide_list(set_focus) {
	if (set_focus)
		edit_profile_name.focus();
	hidden_div.style.display = 'none';
	drop_down_button.setAttribute('src', 'images/drop_down.png');
}

function onBodyClick() {
	if (hidden_div.style.display != 'none') {
		if (!just_shown_list)
			hide_list(false);
		just_shown_list = false;
	}
	if (!just_shown_dialog)
		initDialog(null);
	just_shown_dialog = false;
}

function onKeyPressList(event) {
	if (hidden_div.style.display == 'none')
		return true;
	if (event.keyCode == VK_ESC) {
		hide_list(true);
		return false;
	}
	else if ((event.keyCode == VK_CR) || (event.keyCode == VK_LF)) {
		edit_profile_name.value = list_profile_name.value;
		hide_list(true);
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		hide_list(false)
		return true;
	}
	return true;
}

function onClickList() {
	if (hidden_div.style.display == 'none')
		return true;
	edit_profile_name.value = list_profile_name.value;
	hide_list(true);
}

function onKeyPressEdit(event) {
	if (event.keyCode == VK_ESC) {
		if (hidden_div.style.display != 'none') {
			hide_list(true);
			return false;
		}
	}
	else if (event.keyCode == VK_DOWN) {
		if (hidden_div.style.display == 'none') {
			show_list();
		}
		list_profile_name.focus();
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		if (hidden_div.style.display != 'none') {
			list_profile_name.focus();
			return false;
		}
	}
	else
		hide_list(true);
	return true;
}

function onClickArrow() {
	if (hidden_div.style.display == 'none') {
		show_list();
		list_profile_name.focus();
	}
	else {
		hide_list(true);
	}
}

// Auto-saving user profile functions
function save_and_go(event) {
	var url = this.getAttribute('href');
	// IE fix
	if (!event)
		event = window.event;
	document.configure_test.target = (event.shiftKey ? '_blank' : '');
	document.configure_test.action = 'tests_conf_save.pl?goto=' + url.replace('#', '%23');
	document.configure_test.submit();
	document.configure_test.action = 'tests_conf_save.pl';
	document.configure_test.target = '';
	event.cancelBubble = true;
	return false;
}

function modify_all_links() {
	var links = document.getElementsByTagName('A');
	for (var i = 0; i < links.length; i++) {
		if ((links[i].target == '') && (links[i].href != '') && (links[i].onclick == null)) {
			links[i].onclick = save_and_go;
		}
	}
}

// LSB Profile management functions
function control_profile_list() {
	var elem_v = document.getElementById('spec_version');
	var elem_p = document.getElementById('spec_profile');
	if (!elem_v || !elem_p)
		return;
	if ((elem_v.value == '3.1') || (elem_v.value == '3.2')){
		elem_p.disabled = false;
	}
	else {
		elem_p.disabled = true;
		elem_p.value = 'core,c++,desktop';
	}
}

// Browse for application components
function select_components() {
	var elem;
	elem = document.getElementById('show_hidden_files');
	var show_hidden_files = (elem ? elem.value : '0');
	elem = document.getElementById('last_opened_dir');
	var last_opened_dir = (elem ? elem.value : '/');
	var wnd_sz = getCookie('components_wnd_sz');
	if (!wnd_sz)
		wnd_sz = 'width=550,height=450';
	window.open('choose_components.pl?root=' + encodeURIComponent(last_opened_dir) + '&hidden=' + show_hidden_files, 'select_components', 'scrollbars=yes,resizable=yes,' + wnd_sz).focus();
}

// AJAX
function onSave() {
	if (edit_profile_name.value == '') {
		edit_profile_name.style.borderColor = 'red';
		alert('Please specify the profile name!');
	}
	else if (edit_profile_name.value.search(/[/\\\\<>|\\?\\*]/) != -1) {
		edit_profile_name.style.borderColor = 'red';
		alert('The profile name should contain only symbols that are allowed in file names!');
	}
	else if (app_name.value.search(/[/\\\\<>|\\?\\*]/) != -1) {
		app_name.style.borderColor = 'red';
		alert('The application name should contain only symbols that are allowed in file names!');
	}
	else {
		for (var i=0; i<profiles_list.length; ++i) {
			if (edit_profile_name.value == profiles_list[i]) {
				if (!confirm("Do you want to overwrite the profile '" + edit_profile_name.value + "'?"))
					return;
				break;
			}
		}
		ajax_call_post('action=save_profile', 'configure_test');
	}
}

function onLoad() {
	if (edit_profile_name.value) {
		ajax_call_get('action=load_profile&profile=' + encodeURIComponent(edit_profile_name.value));
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please specify the profile name!');
	}
}

function onDelete() {
	if (edit_profile_name.value) {
		if (confirm("Do you really want to delete the profile '" + edit_profile_name.value + "'?"))
			ajax_call_get('action=delete_profile&profile=' + edit_profile_name.value);
	}
	else {
		edit_profile_name.style.borderColor = 'red';
		alert('Please specify the profile name!');
	}
}

function ajaxProcessResult(responseXML) {
	if (responseXML.getElementsByTagName('profiles').length > 0) {
		var elem_list = document.getElementById('list_profile_name');
		var elem_div = elem_list.parentNode;
		elem_div.removeChild(elem_list);
		elem_list = document.createElement('select');
		elem_list.id = 'list_profile_name';
		elem_list.name = 'list_profile_name';
		elem_list.style.fontFamily = 'monospace';
		elem_list.style.fontSize = '100%';
		elem_list.style.width = '19.7em';
		elem_list.onclick = onClickList;
		elem_list.onkeypress = onKeyPressList;
		elem_div.appendChild(elem_list);
		list_profile_name = elem_list;
		profiles_list = new Array();

		var profile_names = responseXML.getElementsByTagName('profiles')[0].getElementsByTagName('name');
		if (profile_names.length > 0) {
			for (var i=0; i<profile_names.length; ++i) {
				profiles_list[i] = profile_names[i].childNodes[0].nodeValue;
				var tmp_node = document.createElement('option');
				tmp_node.setAttribute('value', profiles_list[i]);
				tmp_node.appendChild(document.createTextNode(profiles_list[i]));
				elem_list.appendChild(tmp_node);
			}
			elem_list.disabled = false;
			if (profile_names.length < 2)
				elem_list.size = 2;
			else if (profile_names.length > 6)
				elem_list.size = 6;
			else
				elem_list.size = profile_names.length;
		}
		else {
			var tmp_node = document.createElement('option');
			tmp_node.setAttribute('value', '');
			tmp_node.appendChild(document.createTextNode('<No profiles present>'));
			elem_list.appendChild(tmp_node);
			elem_list.disabled = true;
			elem_list.size = 2;
		}
	}
	if (responseXML.getElementsByTagName('profile_data').length > 0) {
		document.configure_test.reset();
		var profile_data = responseXML.getElementsByTagName('profile_data')[0].getElementsByTagName('item');
		for (var i=0; i<profile_data.length; ++i) {
			var item_name = profile_data[i].getElementsByTagName('name')[0].childNodes[0].nodeValue;
			var item_values = profile_data[i].getElementsByTagName('value')[0].childNodes;
			var item_value = '';
			if (item_values.length > 0)
				item_value = item_values[0].nodeValue;
			var form_elem = document.getElementById(item_name);
			if (form_elem)
				form_elem.value = item_value;
		}
		control_profile_list();
	}
	edit_profile_name.value = '';
}

// Startup actions
apply_profile_data();
control_profile_list();
modify_all_links();
// ]]>
</script>
DATA

print_footer();
