#!/usr/bin/perl -w

# Application Checker
# Results Page Module (tests_results.pl)
#
# Copyright (C) 2007-2010 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';
use Fcntl ':seek';

autoflush_on();

use Tests_results qw(/./); # import all

#-----------------------------------------------------------------------

if ($_POST{"remove"}) {
	if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
		flock(FILE, LOCK_EX);
		my @list;
		my %decoded = ();
		foreach (keys %_POST) {
			$decoded{unesc_id($_)} = $_POST{$_};
		}
		while (<FILE>) {
			s/[\r\n]//g;
			next if ($_ eq "");
			my $path = $CONFIG{"RESULTS_DIR"}."/$_";
			if (-d $path) {
				if (defined($decoded{$_})) {
					system(esc_cmdline("rm -rf «$path»"));
				}
				else {
					push @list, $_;
				}
			}
		}
		seek(FILE, 0, SEEK_SET);
		truncate(FILE, 0);
		print FILE join("\n", @list)."\n";
		flock(FILE, LOCK_UN);
		close(FILE);
	}
	else {
		print STDERR "Cannot open HISTORY file!\n";
	}
	print 'HTTP/1.0 302 Moved' . CRLF;
	print 'Location: tests_results.pl' . CRLF . CRLF;
	exit;
}

sub http_header {
	my ($add) = @_;
	
	my $res = '';
	
	$res .= 'HTTP/1.0 200 OK' . CRLF;
	$res .= 'Content-type: text/html' . CRLF;
	if ( $add ) {
		$add =~ s/[\r\n]+/${\CRLF}/g;
		$res .= $add;
	}
	$res .= CRLF; # Empty line finishes the header
	
	return $res;
}

sub Summary_cached {
	my ($res_dir, $spec_version) = @_;
	
	my $filename = $res_dir.'/summary.htm';
	
	my $file;
	
	( -f $filename ) # If file already exists
	and open $file, $filename  # and was successfully opened
	and do {
		local $/ = undef; # Enable the "slurp" mode
		my $res = <$file>; # read the file contents
		close $file;
		return $res;
	};
	
	# If can't take the content from cache, than create it
	my $res = Summary( $spec_version );
	
	# And write to the cache file
	open $file, '>'.$filename
	and do {
		print {$file} $res;
		close $file;
	};
	
	return $res;
}

my $link_color = (($appchecker_branch{'STD'} eq 'LSB') ? 'blue' : '#c00000');
my $popup_html_header = <<HEREDOC;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<title>Details</title>

	<style type="text/css">
		body {
			font: small sans-serif;
		}
		a {
			text-decoration: none;
			color: $link_color;
		}
		a:hover {
			text-decoration: underline;
		}
		a[href ^="http://"] {
			background: url(themes/$appchecker_branch{'DIR'}/css/external.png) center right no-repeat;
			target-name: new; target-new: window; target-position: above;
			padding-right: 13px;
		}
		div.box {
			border: 1px solid #aaa;
			background-color: #f9f9f9;
			padding: 5pt;
			padding-left: 15pt;
		}
		table {
			border: 1px #aaaaaa solid;
			border-collapse: collapse;
			background: #f9f9f9;
		}
		table th, table td {
			border: 1px #aaaaaa solid;
			padding: 0.2em 0.4em;
		}
		table th {
			background: #f2f2f2;
		}
		#shadow, #tooltip {
			border: none;
			border-collapse: collapse;
			padding: 0px;
			background: none;
		}
		#shadow td, #tooltip td {
			border: none;
			padding: 0px;
		}
		.title-highlight {
			color: $link_color;
		}
	</style>
	<script type="text/javascript" src="css/app-checker.js"></script>
</head>
<body onclick="javascript:if (typeof(onBodyClick)!='undefined') onBodyClick();">
HEREDOC

my $popup_html_footer = '</body></html>';

if ($_GET{"details"}) {
	my $default_open = !$_GET{'tab'};
	$_GET{'tab'} = 'distrs' if (!$_GET{'tab'});
	my $res_dir = $CONFIG{"RESULTS_DIR"}."/".$_GET{"details"};
	my ($app_name, $host_platform, $spec_version) = read_results_info($res_dir);

	{
		my $err = load_analysis_data($res_dir);

		if ($err) {
			print http_header();
			print_header("App Analysis", "report");
		
			print '<p style="color:red">';
			print $err;
			print '</p>';
			
			# 'View test log' link:
			my $link = '/get.pl'.esc_url($SERVER_PARAM{'APP_DATA'}, {'NOESC' => '/'}).'/results/'.esc_url($_GET{'details'}).'/test_logs/tests_output';
			print '<p><a href="'.esc_html($link).'" target="_blank">View the test log</a></p>';
			
			print '<form name="del_form" action="tests_results.pl" method="post">';
			print '<input type="hidden" name="remove" value="1"/>';
			print '<input type="hidden" name="'.esc_id($_GET{'details'}).'" value="1"/>';
			print '</form>';
			print '<p>[<a href="" onclick="javascript:document.del_form.submit(); return false;">Delete the result</a>]</p>';
			
			print_footer();
			exit 0;
		}
	}
	
	local *header_info_tabs = sub { # Render the header, info block and the tabs ruller.
		return html_title( $app_name, $host_platform, $res_dir )
				.Summary_cached( $res_dir, $spec_version )
				.html_results_tabs( $_GET{'tab'} )
			;
	};

	if ($_GET{"tab"} eq 'distrs') {					# Distributions Tab

		if ( $_GET{"id"} ) {
			my $Did = $_GET{"id"};
			my $show = $_GET{"show"};
			
			print http_header();
			print $popup_html_header;

			print distr_details($Did, $show);

			print $popup_html_footer;

			exit 0;
		}

		my $header_add = "";
		if ( $default_open ) {
			my %cookies_set = ();
			
			# delete all cookies
			$cookies_set{'ifc'} = "";
			$cookies_set{'ifl'} = "";
			$cookies_set{'ifs'} = "";
			$cookies_set{'lfc'} = "";
			$cookies_set{'lfs'} = "";
			$cookies_set{'ds'} = "";
			
			foreach my $key ( sort keys %cookies_set ) {
				my $val = $cookies_set{$key};
				$val = "" if !defined $val;
				my $cookie_pair = $key.'='.esc_url($val);
				$header_add .= "Set-Cookie: ".$cookie_pair."\n";
			}
		}

		# Print header
		print http_header($header_add);
		print_header("App Analysis", "res-distrs");
		print header_info_tabs();

		print D_tab( $spec_version );

		print_footer();
		exit 0;
	} # end of the 'distrs' tab
	elsif ($_GET{"tab"} eq 'com') {					# App Components Tab
		if ( $_GET{"com"} ) {
			my $com = $_GET{"com"};
			
			print http_header();
			print $popup_html_header;
			
			if ( $_GET{"common"} ) {
				print com_common_details($com);
			}
			elsif ( $_GET{"distr"} ) {
				print com_distr_details($com, $_GET{"distr"});
			}
			else {
				print com_details($com);
			}

			print $popup_html_footer;

			exit 0;
		}

		print http_header();
		print_header("App Analysis", "res-com");
		print header_info_tabs();

		print C_tab( $spec_version );

		print_footer();
		exit 0;
	} # end of the 'ints' tab
	elsif ($_GET{"tab"} eq 'libs') {				# Libraries Tab
		if ( $_GET{"lib"} ) {
			my $soname = $_GET{"lib"};
			
			print http_header();
			print $popup_html_header;

			print lib_details($soname, $spec_version);

			print $popup_html_footer;

			exit 0;
		}
		
		# Replace filters in GET request with cookies
		if ( $_GET{'filt_com'} || $_GET{'filt_show'} || $_GET{'nofilt'} ) {
			# set cookies and redirect
			print 'HTTP/1.0 302 Moved' . CRLF;
			
			my %cookies_set = ();
			
			$cookies_set{'lfc'} = $_GET{'filt_com'};
			$cookies_set{'lfs'} = $_GET{'filt_show'};
			$cookies_set{'ds'} = "" if $_GET{'nofilt'};
			$cookies_set{'lf_for'} = $_GET{"details"};
			
			foreach my $key ( sort keys %cookies_set ) {
				my $val = $cookies_set{$key};
				$val = "" if !defined $val;
				my $cookie_pair = $key.'='.esc_url($val);
				print 'Set-Cookie: ' . $cookie_pair . CRLF;
			}
			
			delete $_GET{'filt_com'};
			delete $_GET{'filt_show'};
			delete $_GET{'nofilt'};
			
			my $get = "";
			$get .= join '&', map {esc_url($_).'='.esc_url($_GET{$_})} sort keys %_GET;
			
			print 'Location: tests_results.pl?' . $get . CRLF;
			print CRLF;
			
			exit 0;
		}
		
		if ( $_COOKIE{'lf_for'} && $_COOKIE{'lf_for'} ne $_GET{"details"} ) {
			$_COOKIE{"ds"} = $_COOKIE{'lfc'} = $_COOKIE{'lfs'} = ""; # Cancel cookies
		}
		my $distr_sel = get_distr_sel();
		my $filt_com = $_COOKIE{'lfc'}||"";
		my $filt_show = $_COOKIE{'lfs'}||"";
		
		if ( $_GET{"list"} ) {
		# Library details
			my $soname = $_GET{"list"};

			print http_header();
			print $popup_html_header;

			print lib_distr_list($soname, $distr_sel, $filt_com);

			print $popup_html_footer;

			exit 0;
		}

		# Print the view
		print http_header();
		print_header("App Analysis", "res-libs");
		print header_info_tabs();

		print L_tab( $spec_version, $distr_sel, $filt_com, $filt_show );

		print_footer();
		exit 0;
	}  # end of the 'libs' tab
	elsif ($_GET{"tab"} eq 'ints') {					# Interfaces Tab
		if ( $_GET{"int"} ) {
			print http_header();
			print $popup_html_header;
			
			my $fullname = $_GET{"int"};
			my $lib_key = $_GET{"lib"};
			my $com = $_GET{"com"};
			
			print int_details( $fullname, $lib_key, $com, $spec_version );
			
			print $popup_html_footer;
			exit 0;
		}
		
		# Replace filters in GET request with cookies
		if ( $_GET{'filt_com'} || $_GET{'filt_lib'} || $_GET{'filt_show'} || $_GET{'nofilt'} ) {
			# set cookies and redirect
			print 'HTTP/1.0 302 Moved' . CRLF;
			
			my %cookies_set = ();
			
			$cookies_set{'ifc'} = $_GET{'filt_com'};
			$cookies_set{'ifl'} = $_GET{'filt_lib'};
			$cookies_set{'ifs'} = $_GET{'filt_show'};
			$cookies_set{'ds'} = "" if $_GET{'nofilt'};
			$cookies_set{'if_for'} = $_GET{"details"};
			
			foreach my $key ( sort keys %cookies_set ) {
				my $val = $cookies_set{$key};
				$val = "" if !defined $val;
				my $cookie_pair = $key.'='.esc_url($val);
				print 'Set-Cookie: ' . $cookie_pair . CRLF;
			}
			
			delete $_GET{'filt_com'};
			delete $_GET{'filt_lib'};
			delete $_GET{'filt_show'};
			delete $_GET{'nofilt'};
			
			my $get = "";
			$get .= join '&', map {esc_url($_).'='.esc_url($_GET{$_})} sort keys %_GET;
			
			print 'Location: tests_results.pl?' . $get . CRLF;
			print CRLF;
			
			exit 0;
		}
		
		if ( $_COOKIE{'if_for'} && $_COOKIE{'if_for'} ne $_GET{"details"} ) {
			$_COOKIE{"ds"} = $_COOKIE{'ifc'} = $_COOKIE{'ifl'} = $_COOKIE{'ifs'} = ""; # Cancel cookies
		}
		my $distr_sel = get_distr_sel();
		my $filt_com = $_COOKIE{'ifc'}||"";
		my $filt_lib = $_COOKIE{'ifl'}||"";
		my $filt_show = $_COOKIE{'ifs'}||"";
		
		if ( $_GET{"list"} ) {
			print http_header();
			print $popup_html_header;
			
			my $fullname = $_GET{"list"};
			my $com = $_GET{"com"};
			my $lib_key = $_GET{"lib"};
			
			print int_distr_list( $fullname, $com, $lib_key, $distr_sel, $filt_com, $filt_lib );
			
			print $popup_html_footer;
			exit 0;
		}
		
		print http_header();
		print_header("App Analysis", "res-ints");
		print header_info_tabs();

		print I_tab( $spec_version, $distr_sel, $filt_com, $filt_lib, $filt_show );

		print_footer();
		exit 0;
	} # end of the 'ints' tab
	elsif ($_GET{"tab"} eq 'cert') {					# Certification Tab
		my $name = "test_log.html";
		print http_header();
		print_header("App Analysis", "res-cert");
		if (open(FILE, $CONFIG{"RESULTS_DIR"}."/".$_GET{"details"}."/$name")) {
			my $transfer = 0;
			while (<FILE>) {
				if (m/<body>/) {
					s/^.*<body>\n?//i;
					$transfer = 1;
				}
				if ($transfer) {
					s/(<\/body>|<\/html>)\n?//i;
					if (m/<!-- Test Report Header -->/) {
						print header_info_tabs();
						print <<DATA;
<h1>$appchecker_branch{'STD'} $spec_version Test Results</h1>

<form action="cert_integration.pl?start" method="post" name="integr_data" style="display:none">
<input type="hidden" name="file" value="$CONFIG{'RESULTS_DIR'}/$_GET{'details'}/$_GET{'details'}.tgz" />
</form>
<p style="margin-top: 1.5em;"><a href="cert_integration.pl?start" onclick="javascript:document.integr_data.submit(); return false"><img src="images/medal_small.jpg" width="28" height="28" alt="" /></a> If there are no errors, you can easily <a href="cert_integration.pl?start" onclick="document.integr_data.submit(); return false"><b>apply for certification</b></a> by uploading the results to the <a href="http://$cert_sys_host$cert_sys_base" target="_blank">$appchecker_branch{'STD'} Certification System</a>.</p>

DATA
					}
					else {
						print $_;
					}
				}
			}
			close(FILE);
		}
		else {
			print "<b><font color=\"red\">Sorry, the report '".esc_html($_GET{"details"})."' cannot be opened!</font><br />\nReason: $!</b>\n";
		}
		print_footer();
	} # end of the 'cert' tab
	else {
		print_header("App Analysis", "res-distrs");
		print "<b><font color=\"red\">Invalid tab selected!</font></b>\n";
		print_footer();
	}
} # end of  if $_GET{'details'}
else {
	print http_header();
	print_header("Test Results", "results");

	print "<h1>Test Results</h1>\n";

	print "<form method=\"post\" action=\"tests_results.pl\" name=\"results_list\">\n<table class=\"main\">\n";
	print "  <tr>\n    <td><input type=\"checkbox\" name=\"check_all\" id=\"check_all\" onclick=\"javascript:check_unckeck_all();\" /></td>\n";
	print "    <th style=\"width: 14em;\">Date/Time</th>\n";
	print "    <th style=\"width: 30em;\">Comment</th>\n  </tr>\n";
	my @list = ();
	if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
		flock(FILE, LOCK_EX);
		while(<FILE>) {
			s/[\r\n]//g;
			next if ($_ eq "");
			push @list, $_ if (-d $CONFIG{"RESULTS_DIR"}."/$_");
		}
		seek(FILE, 0, SEEK_SET);
		truncate(FILE, 0);
		print FILE join("\n", @list)."\n";
		flock(FILE, LOCK_UN);
		close(FILE);
		@list = reverse(@list);
		foreach my $name (@list) {
			my $full_name = $CONFIG{"RESULTS_DIR"}."/$name";
			my $href_name = esc_url($name);
			my $id_name = esc_id($name);
			my $result = 'unknown';
			my $app_name = "";
			my @comment = ();
			my $reading_comment = 0;
			if (open(FILE, "$full_name/INFO")) {
				while (<FILE>) {
					s/[\r\n]//g;
					if ($reading_comment) {
						push @comment, esc_html($_);
					}
					else {
						if ($_ eq '') {
							$reading_comment = 1;
						}
						elsif (m/^Verdict: (\S+)/i) {
							if ($1 eq "Success") {
								$result = 'pass';
							}
							elsif ($1 eq "Warnings") {
								$result = 'warn';
							}
							elsif ($1 eq "Failed") {
								$result = 'fail';
							}
						}
						elsif (m/^Application: (.*)$/i) {
							$app_name = esc_html($1);
						}
					}
				}
				close(FILE);
			}
			if (($appchecker_branch{'STD'} eq 'LSB') and open(FILE, "$full_name/INFO_$appchecker_branch{'STD'}")) {
				while (<FILE>) {
					s/[\r\n]//g;
					if (m/^Verdict: (Success|Warnings)/i) {
						$result .= '-'.lc($appchecker_branch{'STD'});
					}
				}
				close(FILE);
			}
			my @extraction = ();
			my $jcomment = join("<br />", @comment);
			if ($jcomment eq "") {
				$jcomment = "&nbsp;";
			}
			my $show_name = $name;
			$show_name =~ s/^.*-([^\-]*)\-([^\-]*)\-([^\-]*)\-([^\-]*)h\-([^\-]*)m\-([^\-]*)s/$3.$2.$1&nbsp;$4:$5:$6/;
			my $upload_icon = "";
			if (! -f $CONFIG{"RESULTS_DIR"}."/$name/test_logs/ftp_upload") {
				$upload_icon = "&nbsp;<a href=\"upload_data.pl?details=$href_name\" title=\"Upload Data to the Linux Foundation\"><img src=\"images/upload_data.png\" alt=\"\" width=\"16\" height=\"16\" /></a>";
			}
			print <<DATA;
  <tr>
    <td rowspan="3"><input type='checkbox' name='$id_name' onclick='javascript:update_state();' /></td>
    <td rowspan="1" style="border-bottom: none;"><img src="images/environment-spacer.gif" width="1" height="16" alt="" /></td>
    <td rowspan="3" valign="top">$jcomment</td>
  </tr>
  <tr>
    <td valign="middle" style="border: none;">
      <table cellpadding="0" cellspacing="0" border="0"><tr>
        <td style="border: none; padding: 0px;"><img src="images/result-$result.png" alt="$result" width="25" height="41" /></td>
        <td style="border: none; padding: 0px;">&nbsp;<a href="tests_results.pl?details=$href_name">$show_name</a><br />&nbsp;Application: $app_name</td>
      </tr></table>
    </td>
  </tr>
  <tr>
    <td style="border: none; text-align: right; vertical-align: bottom;" width="5" height="5">
      <a href="tests_results.pl?details=$href_name&amp;tab=distrs" title="Show distribution compatibility report"><img src="images/res-distrs.png" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$href_name&amp;tab=com" title="Show application components"><img src="images/res-com.png" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$href_name&amp;tab=libs" title="Show required external libraries"><img src="images/res-libs.png" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$href_name&amp;tab=ints" title="Show required external interfaces"><img src="images/res-ints.png" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$href_name&amp;tab=cert" title="Show $appchecker_branch{'STD'} compliance report"><img src="images/res-cert.png" alt="" width="16" height="16" /></a>$upload_icon
    </td>
  </tr>
DATA
		}
	}
	print <<DATA;
</table>
<br /><input type="submit" name="remove" id="remove_button" value="Remove Selected Entries" disabled="disabled" onclick='javascript:return confirm_remove();' />
</form>

<script language="javascript" type="text/javascript">
// <![CDATA[
function count_checked() {
	var num = 0;
	var form = document.results_list;
	for (var i=0; i<form.length; ++i) {
		if ((form[i].type.toLowerCase() == "checkbox") && (form[i].name != "check_all") && form[i].checked) {
			++num;
		}
	}
	return num;
}

function update_state() {
	var elem = document.getElementById("remove_button");
	if (elem) {
		var num = count_checked();
		elem.disabled = (num == 0);
	}
}

function check_unckeck(box, check) {
	temp = box.onchange;
	box.onchange = null;
	box.checked = check;
	box.onchange = temp;
}

function check_unckeck_all() {
	var elem = document.getElementById("check_all");
	if (elem) {
		var checked = elem.checked;
		var form = document.results_list;
		for (var i=0; i<form.length; ++i) {
			if ((form[i].type.toLowerCase() == "checkbox") && (form[i].name != "check_all"))
				check_unckeck(form[i], checked);
		}
		update_state();
	}
}

function confirm_remove() {
	var num = count_checked();
	if (num == 0) {
		alert("Nothing selected!");
		return false
	}
	else
		return confirm("Do you wish to delete " + num + " selected report(s)?");
}
// ]]>
</script>
DATA
	print_footer();
} # end of the result list (result history)

exit 0;

__END__
